"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = __importStar(require("react"));
const git_1 = require("../git");
const CommitBox_1 = require("./CommitBox");
const NewBranchBox_1 = require("./NewBranchBox");
const BranchHeaderStyle_1 = require("../componentsStyle/BranchHeaderStyle");
const typestyle_1 = require("typestyle");
const apputils_1 = require("@jupyterlab/apputils");
const AuthorBox_1 = require("../widgets/AuthorBox");
class BranchHeader extends React.Component {
    constructor(props) {
        super(props);
        this.createNewBranch = (branchName) => __awaiter(this, void 0, void 0, function* () {
            let gitApi = new git_1.Git();
            yield gitApi.checkout(true, true, branchName, false, null, this.props.currentFileBrowserPath);
            this.toggleNewBranchBox();
            this.props.refresh();
        });
        this.toggleNewBranchBox = () => {
            this.props.refresh();
            if (!this.props.disabled) {
                this.setState({
                    showNewBranchBox: !this.state.showNewBranchBox,
                    dropdownOpen: false
                });
            }
            else {
                apputils_1.showErrorMessage('Creating new branch disabled', {
                    message: 'You have staged changes in current branch. Please commit / discard them before creating a new branch.'
                });
            }
        };
        this._previousTopRepoPath = null;
        this.state = {
            dropdownOpen: false,
            showCommitBox: true,
            showNewBranchBox: false
        };
        this.commitAllStagedFiles = this.commitAllStagedFiles.bind(this);
    }
    /** Commit all staged files */
    commitAllStagedFiles(message, path) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (message && message !== '' && (yield this._hasIdentity(path))) {
                    let gitApi = new git_1.Git();
                    const response = yield gitApi.commit(message, path);
                    if (response.ok) {
                        this.props.refresh();
                    }
                }
            }
            catch (error) {
                console.error(error);
                apputils_1.showErrorMessage('Fail to commit', error);
            }
        });
    }
    /** Update state of commit message input box */
    updateCommitBoxState(disable, numberOfFiles) {
        if (disable) {
            if (numberOfFiles === 0) {
                return typestyle_1.classes(BranchHeaderStyle_1.stagedCommitButtonStyle, BranchHeaderStyle_1.stagedCommitButtonDisabledStyle);
            }
            else {
                return typestyle_1.classes(BranchHeaderStyle_1.stagedCommitButtonStyle, BranchHeaderStyle_1.stagedCommitButtonReadyStyle);
            }
        }
        else {
            return BranchHeaderStyle_1.stagedCommitButtonStyle;
        }
    }
    /** Switch current working branch */
    switchBranch(branchName) {
        return __awaiter(this, void 0, void 0, function* () {
            let gitApi = new git_1.Git();
            yield gitApi.checkout(true, false, branchName, false, null, this.props.currentFileBrowserPath);
            this.toggleSelect();
            this.props.refresh();
        });
    }
    toggleSelect() {
        this.props.refresh();
        if (!this.props.disabled) {
            this.setState({
                showCommitBox: !this.state.showCommitBox,
                dropdownOpen: !this.state.dropdownOpen
            });
        }
        else {
            apputils_1.showErrorMessage('Switching branch disabled', {
                message: 'You have staged changes in current branch. Please commit / discard them before switching to another branch.'
            });
        }
    }
    getBranchStyle() {
        if (this.state.dropdownOpen) {
            return typestyle_1.classes(BranchHeaderStyle_1.branchStyle, BranchHeaderStyle_1.expandedBranchStyle);
        }
        else {
            return this.props.showList
                ? BranchHeaderStyle_1.branchStyle
                : typestyle_1.classes(BranchHeaderStyle_1.branchStyle, BranchHeaderStyle_1.smallBranchStyle);
        }
    }
    getHistoryHeaderStyle() {
        if (this.props.sideBarExpanded) {
            return typestyle_1.classes(BranchHeaderStyle_1.openHistorySideBarButtonStyle, BranchHeaderStyle_1.selectedHeaderStyle);
        }
        return typestyle_1.classes(BranchHeaderStyle_1.unSelectedHeaderStyle, BranchHeaderStyle_1.openHistorySideBarButtonStyle);
    }
    getBranchHeaderStyle() {
        if (this.props.sideBarExpanded) {
            return typestyle_1.classes(BranchHeaderStyle_1.branchHeaderCenterContent, BranchHeaderStyle_1.unSelectedHeaderStyle);
        }
        return typestyle_1.classes(BranchHeaderStyle_1.selectedHeaderStyle, BranchHeaderStyle_1.branchHeaderCenterContent);
    }
    render() {
        return (React.createElement("div", { className: this.getBranchStyle() },
            React.createElement("div", { style: { display: 'flex' } },
                React.createElement("div", { className: this.getHistoryHeaderStyle(), onClick: this.props.sideBarExpanded
                        ? null
                        : () => this.props.toggleSidebar(), title: 'Show commit history' },
                    React.createElement("h3", { className: BranchHeaderStyle_1.historyLabelStyle }, "History")),
                React.createElement("div", { className: this.getBranchHeaderStyle(), onClick: this.props.sideBarExpanded
                        ? () => this.props.toggleSidebar()
                        : null },
                    React.createElement("h3", { className: BranchHeaderStyle_1.branchLabelStyle }, this.props.currentBranch),
                    React.createElement("div", { className: this.props.disabled
                            ? typestyle_1.classes(BranchHeaderStyle_1.branchDropdownButtonStyle, BranchHeaderStyle_1.headerButtonDisabledStyle)
                            : BranchHeaderStyle_1.branchDropdownButtonStyle, title: 'Change the current branch', onClick: () => this.toggleSelect() }),
                    !this.state.showNewBranchBox && (React.createElement("div", { className: this.props.disabled
                            ? typestyle_1.classes(BranchHeaderStyle_1.newBranchButtonStyle, BranchHeaderStyle_1.headerButtonDisabledStyle)
                            : BranchHeaderStyle_1.newBranchButtonStyle, title: 'Create a new branch', onClick: () => this.toggleNewBranchBox() })),
                    this.state.showNewBranchBox && this.props.showList && (React.createElement(NewBranchBox_1.NewBranchBox, { createNewBranch: this.createNewBranch, toggleNewBranchBox: this.toggleNewBranchBox })),
                    this.props.upstreamBranch != null &&
                        this.props.upstreamBranch !== '' && (React.createElement("div", { className: BranchHeaderStyle_1.branchTrackingIconStyle })),
                    this.props.upstreamBranch != null &&
                        this.props.upstreamBranch !== '' && (React.createElement("h3", { className: BranchHeaderStyle_1.branchTrackingLabelStyle }, this.props.upstreamBranch)))),
            !this.props.sideBarExpanded && (React.createElement(React.Fragment, null,
                this.state.dropdownOpen && (React.createElement("div", null, this.props.data.map((branch, branchIndex) => {
                    return (React.createElement("li", { className: BranchHeaderStyle_1.branchListItemStyle, key: branchIndex, onClick: () => this.switchBranch(branch.name) }, branch.name));
                }))),
                this.state.showNewBranchBox && (React.createElement("div", null,
                    "Branching from ",
                    this.props.currentBranch)),
                this.state.showCommitBox && this.props.showList && (React.createElement(CommitBox_1.CommitBox, { checkReadyForSubmit: this.updateCommitBoxState, stagedFiles: this.props.stagedFiles, commitAllStagedFiles: this.commitAllStagedFiles, topRepoPath: this.props.topRepoPath, refresh: this.props.refresh }))))));
    }
    /**
     * Does the user have an known git identity.
     *
     * @param path Top repository path
     * @returns Success status
     */
    _hasIdentity(path) {
        return __awaiter(this, void 0, void 0, function* () {
            // If the repository path changes, check the identity
            if (path !== this._previousTopRepoPath) {
                let gitApi = new git_1.Git();
                try {
                    const apiResponse = yield gitApi.config(path);
                    if (apiResponse.ok) {
                        const options = (yield apiResponse.json()).options;
                        const keys = Object.keys(options);
                        // If the user name or email is unknown, ask the user to set it.
                        if (keys.indexOf('user.name') < 0 || keys.indexOf('user.email') < 0) {
                            let result = yield apputils_1.showDialog({
                                title: 'Who is committing?',
                                body: new AuthorBox_1.GitAuthorForm()
                            });
                            if (!result.button.accept) {
                                console.log('User refuses to set his identity.');
                                return false;
                            }
                            const identity = result.value;
                            const response = yield gitApi.config(path, {
                                'user.name': identity.name,
                                'user.email': identity.email
                            });
                            if (!response.ok) {
                                console.log(yield response.text());
                                return false;
                            }
                        }
                        this._previousTopRepoPath = path;
                    }
                }
                catch (error) {
                    throw new Error('Fail to set your identity. ' + error.message);
                }
            }
            return true;
        });
    }
}
exports.BranchHeader = BranchHeader;
