import { IRenderMimeRegistry } from '@jupyterlab/rendermime';
import { IDiffContext } from './model';
import { JupyterFrontEnd } from '@jupyterlab/application';
/**
 * Method to open a main menu panel to show the diff of a given Notebook file.
 * If one already exists, just activates the existing one.
 *
 * @param filePath the path relative to the Git repo root.
 * @param app The JupyterLab application instance
 * @param diffContext the context in which the diff is being requested
 * @param renderMime the renderMime registry instance from the
 * @param topRepoPath the Git repo root path.
 */
export declare function openDiffView(filePath: string, app: JupyterFrontEnd, diffContext: IDiffContext, renderMime: IRenderMimeRegistry, topRepoPath: string): Promise<void>;
/**
 * Gets the path of the file relative to the Jupyter server root. This resolves the server root path after calling
 * the `/git/server_root` REST API, and uses the Git repo root and the file path relative to the repo root to resolve
 * the rest.
 * @param path the file path relative to Git repo root
 * @param topRepoPath the Git repo root path
 */
export declare function getRelativeFilePath(path: string, topRepoPath: string): Promise<string>;
/**
 * Visible for testing.
 */
export declare function clearCache(): void;
